# -*- coding: utf-8 -*-
# This software include silk icon set.
#   http://www.famfamfam.com/lab/icons/silk/
#
from trac.core import *
from trac.config import Option, BoolOption, Configuration
from trac.web.chrome import add_script,add_stylesheet,ITemplateProvider
from trac.web.api import ITemplateStreamFilter
from trac.web.main import IRequestFilter
from trac.util.html import Markup
from trac.util.datefmt import format_date, parse_date

from genshi.filters.transform import Transformer
from genshi.template import MarkupTemplate

import os
import re

# This module provide decoration for ticket fields and report.
# To configurate this plugin, edit trac.ini.
# 
# = trac.ini =
# ..
# [decorator]
# # wheather use iconized report
# iconized_report=True
# # fields list to add calendar feature
# calendar_fields=
#
# # configure icon set for iconized report.
# type.task=decorator/icons/page_edit.png
# type.defect=decorator/icons/bug.png
# type.enhancement=decorator/icons/page_add.png
#
# priority.blocker = decorator/icons/blocker.png
# priority.critical = decorator/icons/critical.png
# priority.major = decorator/icons/major.png
# priority.minor = decorator/icons/minor.png
# priority.trivial = decorator/icons/trivial.png
#
# status.assigned=decorator/icons/user_gray.png
# status.accepted=decorator/icons/user_gray.png
# status.new=decorator/icons/new.png
# status.closed=decorator/icons/tick.png
# status.reopened=decorator/icons/door_out.png
# status.verify=decorator/icons/magnifier.png
#
#
# # configure icon set for iconized report in Japanese.
# type.タスク=decorator/icons/page_edit.png
# type.バグ=decorator/icons/bug.png
# type.不具合=decorator/icons/bug.png
# type.機能追加=decorator/icons/page_add.png
# type.仕様変更=decorator/icons/page_refresh.png
# type.課題=decorator/icons/page_white_error.png
# type.質問=decorator/icons/help.png

# priority.重大 = decorator/icons/blocker.png
# priority.緊急 = decorator/icons/critical.png
# priority.高 = decorator/icons/major.png
# priority.中 = decorator/icons/minor.png
# priority.低 = decorator/icons/trivial.png
#
# status.assigned=decorator/icons/user_gray.png
# status.accepted=decorator/icons/user_gray.png
# status.new=decorator/icons/new.png
# status.closed=decorator/icons/tick.png
# status.reopened=decorator/icons/door_out.png
# status.verify=decorator/icons/magnifier.png

class DecoratorModule(Component):
    implements(ITemplateStreamFilter, ITemplateProvider)

    iconized_report = BoolOption('decorator', 'iconized_report','true',
                             u"""리포트에 아이콘을 표시할지 여부를 설정합니다.
                             
                             이용하려면 별도로 trac.ini 파일 설정이 필요합니다.""")
    calendar_fields = Option('decorator', 'calendar_fields','',u"""입력 시에 달력을 이용할 필드를 설정합니다. (!GanttCalendar 와 마일스톤 관련 필드 제외)""")
    ext_decoration  = BoolOption('decorator', 'ext_decoration','false',u"""셀렉트박스 꾸미기 기능을 이용할지 여부를 설정합니다. (default: false)""")
    cal_ganttcalendar = BoolOption('decorator', 'cal_ganttcalendar','true',u"""!GanttCalendar 플러그인 관련 날짜 입력 시에 달력을 이용할지 여부를 설정합니다.""")
    cal_milestone = BoolOption('decorator', 'cal_milestone','false',u"""마일스톤 완료 관련 날짜 입력 시에 달력을 이용할지 여부를 설정합니다. (default: false)""")
    match_req_regex = Option('decorator', 'match_req_regex','',u"""추가할 match_request 경로(path)를 Regular Expression 으로 설정합니다.""")

    def filter_stream(self, req, method, filename, stream, data):
        match_req_regex = Configuration.get(self.config,"decorator",'match_req_regex','');
        match_req_regex = match_req_regex and '|'+ match_req_regex or ''

        if re.match(r'^/(ticket|newticket|report/|query|milestone/|admin/)'+ match_req_regex, req.path_info):
            calendar_fields = Configuration.getlist(self.config,"decorator","calendar_fields", "");
            iconized_report = Configuration.getbool(self.config,"decorator","iconized_report", "True");
            ext_decoration = Configuration.getbool(self.config,"decorator","ext_decoration", "False");
            cal_ganttcalendar = Configuration.getbool(self.config,"decorator","cal_ganttcalendar", "True");
            cal_milestone = Configuration.getbool(self.config,"decorator","cal_milestone", "False");

            if not (cal_ganttcalendar or cal_milestone or calendar_fields) \
              and not (iconized_report and re.match(r'^/(ticket|report/|query)'+ match_req_regex, req.path_info)) \
              and not (ext_decoration & (not re.match(r'^/query',req.path_info))):
                return stream

            add_stylesheet(req, 'decorator/js/jqueryui/themes/base/ui.all.css');
            add_stylesheet(req, 'decorator/js/jqueryui/themes/ui-lightness/jquery-ui-1.7.1.custom.css');
            add_script(req, 'decorator/js/jqueryui/ui.core.js');
            add_script(req, 'decorator/js/jqueryui/ui.datepicker.js');
            add_script(req, 'decorator/js/jqueryui/i18n/ui.datepicker-ko.js');

            # avoid query does not work with ext.js problem.
            if ext_decoration & (not (filename=="query.html" or filename == "gantt.html" or filename=="calendar.html")):
                add_stylesheet(req, 'decorator/js/ext/resources/css/form.css');
                add_stylesheet(req, 'decorator/js/ext/resources/css/combo.css');
                add_script(req, 'decorator/js/ext/adapter/ext/ext-base.js');
                add_script(req, 'decorator/js/ext/adapter/jquery/jquery-plugins.js');
                add_script(req, 'decorator/js/ext/ext-all.js');
            add_script(req, 'decorator/js/trac-decorator.js');

            script = "<script type='text/javascript'>\n"
            script = script+"window.onload = function(){\n"
            script = script+"  ultimania.trac.base = '" + req.href.chrome('decorator') + "';\n"

            script += self.set_datepicker(req)

            if cal_ganttcalendar and filename == 'ticket.html':
                script += self.set_datepicker(req, class_name='ganttcalendar')
                calendar_fields = ['due_assign', 'due_close'] + calendar_fields
            elif cal_ganttcalendar and filename in ('gantt.html', 'admin_holiday.html'):
                calendar_fields = ['baseday', 'date']
            elif cal_milestone and filename in ('milestone_edit.html', 'admin_milestones.html'):
                calendar_fields = ['duedate', 'completeddate']

            def datepicker_callback(name, event):
                attrs = event[1][1]
                for field in attrs:
                    if field[0] == 'name':
                      if field[1] in ('field_due_assign', 'field_due_close'):
                        class_name = 'ganttcalendar'
                      else:
                        class_name = 'decorator'
                      break;
                return ' '.join(filter(None, (attrs.get('class'), class_name)))
            ########################

            for field in calendar_fields:
                if filename == 'ticket.html':
                    field = "field_" + field
                stream |= Transformer('//input[@name="' + field + '"]').attr('class', datepicker_callback)

            if iconized_report and re.match(r'^/(ticket|report/|query)'+ match_req_regex, req.path_info):
                if filename == 'ticket.html':
                    def complete_callback(name, event):
                        attrs = event[1][1]
                        return ' '.join(filter(None, (attrs.get('class'), 'complete')))
                    stream |= Transformer('//td[@headers="h_complete"]').attr('class', complete_callback)
                else:
                    config = self.config['decorator']
                    for (value, icon) in config.options():
                        try:
                            idx = value.index('.')
                            field = value[0:idx]
                            name = value[idx+1:]
                            script = script+self.generate_decorator(field ,name ,icon ,req)
                        except ValueError:
                            None
                script = script+"  ultimania.trac.decorate();\n"

            script = script+"}\n"
            if ext_decoration & (not re.match(r'^/query',req.path_info)):
                script = script+"  Ext.BLANK_IMAGE_URL = '"+req.href.chrome('decorator/js/ext/resources/images/default/s.gif')+"';\n"
                script = script+"  ultimania.trac.initExt();\n"
            script = script+"</script>"
            return stream | Transformer('//div[@id="footer"]').before(MarkupTemplate(script).generate())
        else:
            return stream

    def generate_decorator(self,fieldname, name, icon, req):
        return "  ultimania.trac.registDecorator('"+fieldname+"','"+name+"','"+req.href.chrome(icon)+"');\n"

    def get_templates_dirs(self):
        from pkg_resources import resource_filename
	return []

    def get_htdocs_dirs(self):
        from pkg_resources import resource_filename
        return [('decorator', resource_filename(__name__, 'htdocs'))]

    # set datepicker
    def set_datepicker(self, req, class_name='decorator'):

        def get_dateFormat():
            dateFormat = 'yy-mm-dd'
            if class_name == 'ganttcalendar':
                format = self.config['ganttcalendar'].get('format', default='%Y/%m/%d') or '%Y/%m/%d'
                dateFormat = format.replace('%Y', 'yy', 1).replace('%y', 'yy', 1) \
                                   .replace('%m', 'mm', 1).replace('%d', 'dd', 1)
            else:
                sample_date = '1999-10-29'
                locale_date = format_date(parse_date( sample_date, tzinfo=req.tz))
                try:
                    parse_date( locale_date)
                    dateFormat = locale_date.replace('1999', 'yy', 1).replace('99', 'yy', 1) \
                                            .replace('10', 'mm', 1).replace('29', 'dd', 1)
                except:
                    pass
            return dateFormat
        ########################

        def get_firstDay():
            return str( self.config['ganttcalendar'].getint('first_day', default=0))
        ########################

        return "  $(\"." + class_name + "\").datepicker({ dateFormat: '" + get_dateFormat() + "'" \
                + ", firstDay: " + get_firstDay() \
                + ", buttonImage: '" + req.href.chrome('common', 'ics.png') + "'});\n"
